/*
   $Id: GlintEngineFreq.h 703 2013-01-25 09:06:49Z tfaraut $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

/* Deprecated : using SuitableRegion.* instead */

#ifndef __GLINTENGINEFREQUENCY_H__
#define __GLINTENGINEFREQUENCY_H__

#include "buffer.h"
#include "GlintEngine.h"
#include "Display.h"
#include <stack>

enum FREQ
	{
	LOW = 0,
	HIGH = 1,
	CONDEMNED = 2
	};

typedef unsigned char freqCode;


/** \brief the Comparison hierarchy 
 *         - The aim is to generate design oligonucleotide primer
 *				for FISH experiment
 * 		   - First of all we generate an Index Tree
 * 		   - Then we select non frequent probes from a query
 * 		   - A second selection gives regions suitable to be hybridization probes.
 * 		   - Finally we select primers for PCR considering Tm and %GC
*/


/** \brief The base abstract object
*/
class GlintEngineFreq: public GlintEngine {
	
	public :
		//Constructeur
		GlintEngineFreq(QueryBuffer*, Display& d);
		//Destructeur
		~GlintEngineFreq()
		{		
		}
		void Run();
		
		size_t         GetBufferSize()      { return frequentCodeBuffer.size(); };
		const Buffer&  GetFrequentBuffer()  { return frequentCodeBuffer; };	
		freqCode*      GetBufferBase()      { return (freqCode*) frequentCodeBuffer.GetBase(); };
		
		const vector<regionFreq>&  GetSuitableRegions()  {return __suitableRegions;};		
		unsigned char 	Decode(freqCode f);
		
	protected :	
			 
		bool _BufferPut(freqCode f) { return frequentCodeBuffer.Put((char) f); };
			
	private:   //Methods
		void __MaskRepeats();
		void __SearchingForSeeds();
		void __DetectSuitableRegions();	
		void __SentencingRegions();
		void __SelectingSuitableRegions();
		void __MaskSentencedRegions();
		 
		void __AddSuitableRegion( uint32_t start, uint32_t end ); 
		 
		uint32_t __max0( int );
		bool IsLower( const unsigned char ) const;
		
	private :	
		Buffer               frequentCodeBuffer;     // Buffer containing the recoded sequence : LOW, HIGH or CONDEMNED (see enum FREQ)	
		
		double SentenceScore[3];
		double SuitableScore[3];

		vector<regionFreq>  __sentencedRegions;
		vector<regionFreq>  __suitableRegions;	
		
	//For debugging only	
	protected :			
		void _DisplayFrequentBuffer();	
			
};

inline uint32_t GlintEngineFreq::__max0( int v )
{
	return ( v > 0 ) ? v : 0;
}

inline bool GlintEngineFreq::IsLower(const unsigned char c) const { return ( (c&0x20) != 0) ? true : false; }

inline unsigned char GlintEngineFreq::Decode(freqCode f)
{
	unsigned char c;
	
	switch ( f ) 
	{
		case HIGH      : c = 'H'; break;
		case LOW       : c = 'L'; break;
		case CONDEMNED : c = 'C'; break;
		default        : c = '?'; break;
	}
	
	return c;
}

#endif
